import pytest
import glob
from abcli.plugins.graphics.gif import generate_animated_gif
from abcli.plugins.testing import download_object
from abcli.modules import objects


@pytest.mark.parametrize(
    ["object_name"],
    [
        ["void"],
        ["2024-05-07-18-45-13-31678"],
    ],
)
def test_generate_animated_gif(object_name: str):
    assert download_object(object_name)

    list_of_images = list(glob.glob(objects.path_of("*.png", object_name)))
    if object_name != "void":
        assert list_of_images

    assert generate_animated_gif(
        list_of_images,
        objects.path_of("test.gif", object_name),
    )



import os

NAME = "abcli"

VERSION = "8.184.1"

DESCRIPTION = "🚀 a language to speak AI."


def fullname() -> str:
    abcli_git_branch = os.getenv("abcli_git_branch", "")
    return "abcli-{}{}".format(
        VERSION,
        f"-{abcli_git_branch}" if abcli_git_branch else "",
    )



import argparse
from . import *
from abcli.logger import logger


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="read",
    help="clone,create,read,search,write",
)
parser.add_argument(
    "--all",
    default=0,
    type=int,
)
parser.add_argument(
    "--dataframe",
    default=0,
    type=int,
)
parser.add_argument(
    "--destination",
    type=str,
    default="",
)
parser.add_argument(
    "--keyword",
    type=str,
)
parser.add_argument(
    "--like",
    default=0,
    type=int,
)
parser.add_argument(
    "--source",
    type=str,
    default="",
)
parser.add_argument(
    "--unique",
    default=1,
    type=int,
    help="cache.read('unique')",
)
parser.add_argument(
    "--value",
    type=str,
    default="unknown",
)
args = parser.parse_args()

success = False
if args.task == "clone":
    success = clone(args.source, args.destination)
elif args.task == "create":
    success = create()
elif args.task == "read":
    output = read(
        args.keyword,
        all=args.all,
        like=args.like,
        dataframe=args.dataframe,
        unique=args.unique,
    )

    print(
        ("None" if output.empty else output.drop(columns=["log_id"]))
        if args.dataframe
        else output
    )
    success = True
elif args.task == "search":
    for keyword, value in search(args.keyword).items():
        print(f"{keyword}: {value}")
    success = True
elif args.task == "write":
    success = write(args.keyword, args.value)
else:
    logger.error(f"-{NAME}: {args.task}: command not found.")

if not success:
    logger.error(f"-{NAME}: {args.task}: failed.")



from .keywords import KEYWORDS


def pack(keyword):
    """pack keyword.

    Args:
        keyword (str): keyword.

    Returns:
        str: packed keyword.
    """
    return {v: k for k, v in KEYWORDS.items()}.get(keyword, keyword)



import argparse
from abcli import env
from . import *

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="",
    help="create_bucket|download_file|exists|list_of_objects|upload_file",
)
parser.add_argument(
    "--filename",
    type=str,
    default="",
)
parser.add_argument(
    "--bucket_name",
    type=str,
    default=env.abcli_aws_s3_bucket_name,
)
parser.add_argument(
    "--count",
    type=int,
    default=9999,
)
parser.add_argument(
    "--delim",
    type=str,
    default=", ",
)
parser.add_argument(
    "--item_name",
    default="object",
    type=str,
)
parser.add_argument(
    "--log",
    default=1,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--object_name",
    type=str,
    default="",
)
parser.add_argument(
    "--postfix",
    type=str,
    default="",
)
parser.add_argument(
    "--prefix",
    type=str,
    default="",
)
parser.add_argument(
    "--recursive",
    type=int,
    default=1,
    help="0|1",
)

args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = True
output = None
if args.task == "create_bucket":
    success = Storage(args.bucket_name).s3 is not None
elif args.task == "download_file":
    success = Storage(args.bucket_name).download_file(
        bucket_name=args.bucket_name,
        object_name=args.object_name,
        filename=args.filename,
    )
elif args.task == "exists":
    print(Storage(args.bucket_name).exists(args.object_name))
    success = True
elif args.task == "list_of_objects":
    output = Storage(args.bucket_name).list_of_objects(
        count=args.count,
        postfix=args.postfix,
        prefix=args.prefix,
        recursive=args.recursive,
    )
    success = True
elif args.task == "upload_file":
    success = Storage(args.bucket_name).upload_file(
        filename=args.filename,
        object_name=args.object_name,
        bucket_name=args.bucket_name,
    )
else:
    print(f"-{NAME}: {args.task}: command not found.")

if success and output is not None:
    if args.log:
        print(f"{len(output):,} {args.item_name}(s): {delim.join(output)}")
    else:
        print(delim.join(output))

if not success:
    print(f"-{NAME}: {args.task}: failed.")



import argparse
from abcli import string, NAME
from abcli.file import size
from abcli.file import load_text, save_text
from abcli.logger import logger

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="size",
)
parser.add_argument(
    "--filename",
    type=str,
)
parser.add_argument(
    "--this",
    type=str,
)
parser.add_argument(
    "--that",
    type=str,
)
args = parser.parse_args()

success = False
if args.task == "replace":
    logger.info(f"{NAME}.{args.task}: {args.this} -> {args.that} in {args.filename}")
    success, content = load_text(args.filename)
    if success:
        content = [line.replace(args.this, args.that) for line in content]

        success = save_text(args.filename, content)
elif args.task == "size":
    print(string.pretty_bytes(size(args.filename)))
    success = True
else:
    logger.error(f"-{NAME}: {args.task}: command not found.")

if not success:
    logger.error(f"-{NAME}: {args.task}: failed.")



import argparse
from abcli.plugins.git import increment_version, NAME
from abcli.logger import logger


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="",
    help="increment_version",
)
parser.add_argument(
    "--repo_path",
    type=str,
)
parser.add_argument(
    "--verbose",
    type=int,
    default="0",
    help="0|1",
)
args = parser.parse_args()

success = False
if args.task == "increment_version":
    success = increment_version(
        repo_path=args.repo_path,
        verbose=args.verbose == 1,
    )
else:
    logger.error(f"-{NAME}: {args.task}: command not found.")

if not success:
    logger.error(f"-{NAME}: {args.task}: failed.")



import os
import os.path
import time
from abcli import env, file
from abcli import path
from abcli import env
from abcli import string
from abcli.logger import logger, crash_report


class Storage:
    def __init__(self, bucket_name=env.abcli_aws_s3_bucket_name):
        self.region = env.abcli_aws_region

        try:
            import boto3

            self.s3 = boto3.client("s3", region_name=self.region)
        except:
            crash_report("-storage: failed.")

        self.bucket_name = bucket_name

        self.create_bucket()

    def create_bucket(self, bucket_name=None):
        """create bucket.

        Args:
            bucket_name (str, optional): bucket name. Defaults to None.

        Returns:
            bool: success.
        """
        if bucket_name is None:
            bucket_name = self.bucket_name

        try:
            import boto3

            if boto3.resource("s3").Bucket(bucket_name).creation_date is not None:
                logger.debug(f"-storage: create_bucket: {bucket_name}: already exists.")
                return True

            self.s3.create_bucket(
                Bucket=bucket_name,
                CreateBucketConfiguration={"LocationConstraint": self.region},
            )
        except:
            crash_report(f"-storage: create_bucket: {bucket_name}: failed.")
            return False

        return True

    def download_file(
        self,
        object_name,
        filename="",
        bucket_name=None,
        civilized=False,
        log=True,
        overwrite=False,
    ):
        """download file.

        Args:
            object_name (str): object name.
            filename (str, optional): filename to download. Defaults to "".
            bucket_name (str, optional): bucket name. Default to None.
            civilized (bool, optional): if failed, do not print error message. Defaults to False.
            log (bool, optional): log. Default to False.
            overwrite (bool, optional): overwrite filename. Defaults to False.

        Returns:
            bool: success
        """
        if filename == "static":
            filename = os.path.join(
                env.abcli_path_static,
                object_name.replace("/", "-"),
            )

        if filename == "object":
            filename = os.path.join(
                env.abcli_object_root,
                "/".join(object_name.split("/")[1:]),
            )

        if not overwrite and file.exist(filename):
            return True

        if not path.create(file.path(filename)):
            return False

        if bucket_name is None:
            bucket_name = self.bucket_name

        success = False
        try:
            self.s3.download_file(bucket_name, object_name, filename)
            success = True
        except:
            if not civilized:
                crash_report(
                    f"-storage: download_file({bucket_name}/{object_name}): failed."
                )

        if success and log:
            logger.info(f"downloaded {bucket_name}/{object_name} -> {filename}")

        return success

    def list_of_objects(
        self,
        prefix,
        bucket_name=None,
        count=-1,
        postfix="",
        recursive=True,
        include_folders=False,
    ):
        """list objects.

        Args:
            prefix (str): prefix.
            bucket_name (str, optional): bucket name. Defaults to None.
            count (int, optional): count to return. Defaults to -1.
            postfix (str, optional): postfix. Defaults to "".
            recursive (bool, optional): recursive. Defaults to True.
            include_folders (bool, optional): include folders. Defaults to False.

        Returns:
            List[str]: list of objects.
        """
        prefix = f"{env.abcli_aws_s3_prefix}/{prefix}"

        if bucket_name is None:
            bucket_name = self.bucket_name

        output = []
        try:
            import boto3

            output = [
                string.after(object_summary.key, prefix)
                for object_summary in boto3.resource("s3")
                .Bucket(bucket_name)
                .objects.filter(Prefix=prefix)
                # .limit(count)
            ]
        except:
            crash_report("-storage: list_of_objects: failed.")

        output = [thing[1:] if thing.startswith("/") else thing for thing in output]

        if include_folders:
            output = sorted(list({thing.split("/")[0] for thing in output}))
        elif not recursive:
            output = [thing for thing in output if "/" not in thing]

        if postfix:
            output = [thing for thing in output if thing.endswith(postfix)]

        if count != -1:
            output = output[:count]

        return output

    def exists(
        self,
        object_name,
        bucket_name=None,
    ):
        """does object_name exist?

        Args:
            object_name (str): object name.
            bucket_name (str, optional): bucket name. Defaults to None.

        Returns:
            bool: success
        """
        if bucket_name is None:
            bucket_name = self.bucket_name

        import boto3
        import botocore

        try:
            boto3.resource("s3").Object(
                bucket_name, "/".join([env.abcli_aws_s3_prefix, object_name])
            ).load()
        except botocore.exceptions.ClientError as e:
            if e.response["Error"]["Code"] != "404":
                crash_report("-storage: exists: failed.")
            return False

        return True

    def upload_file(
        self,
        filename,
        object_name=None,
        bucket_name=None,
        overwrite=False,
    ):
        """upload file.

        Args:
            filename (str): filename.
            object_name (str, optional): object name. Defaults to None.
            bucket_name (str, optional): bucket name. Defaults to None.
            overwrite (bool, optional): overwrite. Defaults to False.

        Returns:
            bool: success.
            str: bucket_name.
            str: object_name.
        """
        if bucket_name is None:
            bucket_name = self.bucket_name

        if not filename:
            logger.warning("-storage: upload_file: no file.")
            return False, bucket_name, ""

        if object_name is None:
            object_name = "{}/{}{}".format(
                env.abcli_aws_s3_prefix,
                env.abcli_object_name,
                (
                    string.after(filename, env.abcli_object_name)
                    if env.abcli_object_name in filename
                    else filename
                ),
            )

        success = True
        if not self.exists(object_name) or overwrite:
            time_ = time.time()
            try:
                self.s3.upload_file(filename, bucket_name, object_name)
            except:
                crash_report("-storage: upload_file: failed.")
                success = False

            if success:
                logger.info(
                    "uploaded {}:{} -> {}/{}: {}.".format(
                        filename,
                        string.pretty_bytes(file.size(filename)),
                        bucket_name,
                        object_name,
                        string.pretty_duration(
                            time.time() - time_,
                            include_ms=True,
                            short=True,
                        ),
                    )
                )

        return success, bucket_name, object_name

    def url(self, object_name, filename):
        """return url for object_name/filename.

        Args:
            object_name (str): object name.
            filename (str): filename.

        Returns:
            bool: success.
        """
        return "https://{}.s3.{}.amazonaws.com/{}/{}/{}".format(
            self.bucket_name,
            self.region,
            env.abcli_aws_s3_prefix,
            object_name,
            filename,
        )



import datetime
import json


# https://gist.github.com/jsbueno/9b2ea63fb16b84658281ec29b375283e
class JsonEncoder(json.JSONEncoder):
    def default(self, obj):
        try:
            return super().default(obj)
        except TypeError:
            pass

        if obj.__class__.__name__ == "ndarray":
            return obj.tolist()

        if isinstance(obj, datetime.datetime):
            return "{}".format(obj)

        return (
            obj.__dict__
            if not hasattr(type(obj), "__json_encode__")
            else obj.__json_encode__
        )



import os
from abcli import env, path


def list_of_external(repo_names=False):
    """return list of external plugins.

    Args:
        repo_names (bool, optional): return repo names. Defaults to False.

    Returns:
        list[str]: list of external plugins.
    """

    output = sorted(
        [
            repo_name
            for repo_name in [
                path.name(path_)
                for path_ in path.list_of(env.abcli_path_git)
                if path.exist(os.path.join(path_, ".abcli"))
            ]
            if repo_name != "awesome-bash-cli"
        ]
    )

    if not repo_names:
        output = [repo_name.replace("-", "_") for repo_name in output]

    return output
